package de.urszeidler.eclipse.callchain.generatorservice;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Generatorservice extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "de.urszeidler.eclipse.callchain.generatorservice";

	private Map<String, GeneratorDescriptor> generatorRegistry = null;// new
	private Map<String, Category> categorys = new HashMap<String, Category>();
	// HashMap
	// <
	// String
	// ,
	// GeneratorDescriptor
	// >();
	// The shared instance
	private static Generatorservice plugin;

	public static String ID = PLUGIN_ID;

	/**
	 * The constructor
	 */
	public Generatorservice() {

	}

	/**
	 * extract the raw configurations
	 * 
	 * @return
	 */
	private List<GeneratorDescriptor> listInstalledGenerators() {
		if (generatorRegistry == null) {
			generatorRegistry = new HashMap<String, GeneratorDescriptor>();
		}
		List<GeneratorDescriptor> list = new ArrayList<GeneratorDescriptor>();
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IConfigurationElement[] extensions = reg.getConfigurationElementsFor(ID);
		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement element = extensions[i];
			if (element.getName().equals("category")) {
				Category category = new Category();
				category.setId(element.getAttribute("id"));
				category.setName(element.getAttribute("name"));
				category.setDescription(element.getAttribute("description"));
				categorys.put(category.getId(), category);

			}
		}

		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement element = extensions[i];
			if (element.getName().equals("atlflowgenerator")) {
				GeneratorDescriptor descriptor = new GeneratorDescriptor();
				descriptor.setId(element.getAttribute("id"));
				descriptor.setName(element.getAttribute("typeName"));
				descriptor.setSmallIconPath(element.getAttribute("smallIcon"));
				descriptor.setLargeIconPath(element.getAttribute("largeIcon"));
				descriptor.setColor(element.getAttribute("color"));
				descriptor.setClassName(element.getAttribute("className"));
				descriptor.setDescription(element.getAttribute("description"));
				descriptor.setCategory(element.getAttribute("category"));
				descriptor.setConstributor(element.getContributor().getName());

				generatorRegistry.put(descriptor.getId(), descriptor);
				Category category = categorys.get(descriptor.getCategory());
				if (category == null) {
					category = getCategorys().get("default");
					if (category == null)
						category = new Category("default", "default category");
					categorys.put(category.getId(), category);
				}
				category.getDescriptors().add(descriptor);

				list.add(descriptor);
			}
		}
		return list;
	}

	/**
	 * only available generators will be returnt.
	 * 
	 * @return
	 */
	public List<GeneratorDescriptor> getAvailableGenerators() {

		List<GeneratorDescriptor> list = new ArrayList<GeneratorDescriptor>();
		List<GeneratorDescriptor> installedGenerators = listInstalledGenerators();
		for (GeneratorDescriptor descriptor : installedGenerators) {

			try {
				String constributor = descriptor.getConstributor();
				Bundle bundle = Platform.getBundle(constributor);
				Class<?> clazz = bundle.loadClass(descriptor.getClassName());
				if (GeneratorExecutable.class.isAssignableFrom(clazz))
					descriptor.setGeneratorClass((Class<? extends GeneratorExecutable>) clazz);
				// System.out.println(clazz);
				list.add(descriptor);

			} catch (Exception e) {
				System.err.println(descriptor.getClassName() + "not found");
			}
		}

		return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext
	 * )
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;

		List<GeneratorDescriptor> availableGenerators = getAvailableGenerators();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext
	 * )
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		Collection<GeneratorDescriptor> generatorDescriptors = generatorDescriptors();
		for (GeneratorDescriptor descriptor : generatorDescriptors) {
			try {
				if (descriptor.getGeneratorColor() != null)
					descriptor.getGeneratorColor().dispose();

			} catch (Exception e) {
				System.err.println(e);
			}
		}
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static Generatorservice getDefault() {
		return plugin;
	}

	public URL getInstallURL() {
		return getDefault().getBundle().getEntry("/"); //$NON-NLS-1$
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path.
	 * 
	 * @generated
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public ImageDescriptor getBundledImageDescriptor(String bundelId, String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(bundelId, path);
	}

	public Image getBundledImage(String bundelId, String path) {
		Image image = getImageRegistry().get(bundelId + path);
		if (image == null) {
			ImageDescriptor imageDescriptor = getBundledImageDescriptor(bundelId, path);
			if (imageDescriptor != null) {
				getImageRegistry().put(bundelId + path, imageDescriptor);
				image = getImageRegistry().get(bundelId + path);
			}
		}
		return image;
	}

	public GeneratorDescriptor getGenerator(String id) {
		if (generatorRegistry == null) {
			generatorRegistry = new HashMap<String, GeneratorDescriptor>();
			getAvailableGenerators();
		}
		return generatorRegistry.get(id);
	}

	public Set<String> generatorIds() {
		if (generatorRegistry == null) {
			generatorRegistry = new HashMap<String, GeneratorDescriptor>();
			getAvailableGenerators();
		}
		return generatorRegistry.keySet();
	}

	public Collection<GeneratorDescriptor> generatorDescriptors() {
		if (generatorRegistry == null) {
			generatorRegistry = new HashMap<String, GeneratorDescriptor>();
			getAvailableGenerators();
		}
		return generatorRegistry.values();
	}

	public Map<String, Category> getCategorys() {
		return categorys;
	}

	private void initaliseCategorys() {
		// TODO Auto-generated method stub

	}

}
