package de.urszeidler.eclipse.callchain.generatorservice.executables;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;

public abstract class AbstractExecuteable implements GeneratorExecutable{

	protected static final String URI_CONSTANT = "uri:";
	protected Generic_Generator generator;

	public void configure(Generic_Generator generator) {
		this.generator = generator;
	}

	public String getName() {
		return generator.getName();
	}

	/**
	 * helper function to create a resourceset
	 * 
	 * @return
	 */
	public static ResourceSet createResourceSet() {
		ResourceSet result = new ResourceSetImpl();
		result.getURIConverter().getURIMap().putAll(EcorePlugin.computePlatformURIMap());
		return result;
	}
	/**
	 * @param model
	 * @return
	 */
	protected Object loadEObject(Model model,ResourceSet resourceSet) {
		URI uri = artifactToUri(model);

		// resourceSet.getURIConverter().getURIMap().putAll(
		// EcorePlugin.computePlatformURIMap());
		Resource resource = internalLoad(resourceSet, uri);

		if (resource.getContents().size() == 0)
			return null;

		if (resource.getContents().size() == 1)
			return resource.getContents().get(0);
		else {
			ArrayList<Object> list = new ArrayList<Object>();
			list.addAll(resource.getContents());
			return list;
		}

		// return null;
	}

	/**
	 * @param artifact
	 * @return
	 */
	protected URI artifactToUri(Artifact artifact) {
		String strUri = artifact.getUri();
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		IFile currentFile = wksroot.getFile(new Path(strUri));

		URI uri = fileURI(currentFile);
		return uri;
	}

	/**
	 * @param model
	 * @return
	 */
	protected Object loadEObject(Model model) {
		return loadEObject(model, createResourceSet());
	}

	protected static Resource internalLoad(ResourceSet resourceSet, URI uri) {
		//Diagnostic diagnostic = Diagnostic.OK_INSTANCE;
		Resource resource = resourceSet.createResource(uri);
		if (resource == null)
			return null;

		//Exception rootException = null;
		try {
			resource.load(resourceSet.getLoadOptions());
		} catch (IOException e) {
			//rootException = e instanceof Resource.IOWrappedException ? (Exception) e.getCause() : e;
			//TODO : throw a corexception
			// include only non diagnostic exception in resource.getErrors()
			// if(!(e instanceof Resource.Diagnostic)) {
			//resource.getErrors().add(ModelLoadHelper.createDiagnostic(resource
			// , rootException));
			// }
		} catch (RuntimeException e) {
			//resource.getErrors().add(ModelLoadHelper.createDiagnostic(resource
			// , e));
		}
		// EList<Resource.Diagnostic> errors = resource.getErrors();
		// EList<Resource.Diagnostic> warnings = resource.getWarnings();
		// if (!errors.isEmpty() || !warnings.isEmpty()) {
		// Diagnostic resourceDiagnostic = EcoreUtil.computeDiagnostic(
		// resource, true);
		// Integer severityOpt = new Integer(
		// resourceDiagnostic.getSeverity() == Diagnostic.ERROR ? 0
		// : 1);
		// // String message =
		// // MessageFormat.format(Messages.modelLoadedWithProblems,
		// // new Object[] { severityOpt, resource.getURI() });
		//
		// Object[] data = (rootException != null) ? new Object[] {
		// rootException, resource } : new Object[] { resource };
		// // BasicDiagnostic loadDiagnostic = new
		// // BasicDiagnostic(DIAGNOSTIC_SOURCE, resourceDiagnostic.getCode(),
		// // message, data);
		// // loadDiagnostic.addAll(resourceDiagnostic);
		// // diagnostic = loadDiagnostic;
		// }
		return resource;
	}

	/**
	 * return the first model conforming to one of the metamodel with the uris
	 * 
	 * @param artifacts
	 * @param mmUri
	 * @return
	 */
	protected Model findModelByMetaModel(Collection<Artifact> artifacts, String... mmUri) {
		for (String string : mmUri) {
			Model model = findModelByMetaModel(artifacts, string);
			if (model != null)
				return model;
		}
		return null;
	}

	/**
	 * return the model, if one, conforms to the metamodel with this uri
	 * 
	 * @param artifacts
	 * @param mmUri
	 * @return the model conforms to the metamodel
	 */
	protected Model findModelByMetaModel(Collection<Artifact> artifacts, String mmUri) {
		if (artifacts != null && mmUri != null)
			for (Artifact artifact : artifacts) {
				if (artifact instanceof Model) {
					Model model = (Model) artifact;
					if (model.getMm() != null)
						if ((mmUri.equals(model.getMm().getUri()))
								|| ((URI_CONSTANT + mmUri).equals(model.getMm().getUri())))
							return model;
				}
			}
		return null;
	}

	/**
	 * Creates a new resource set and resource for this file.
	 * 
	 * @param modelFile
	 * @return
	 */
	protected Resource createNewResource(IFile modelFile) {
		ResourceSet resourceSet = createResourceSet();
		return createNewResource(modelFile, resourceSet);
	}

	/**
	 * @param modelFile
	 * @param resourceSet
	 * @return
	 */
	protected Resource createNewResource(IFile modelFile, ResourceSet resourceSet) {
		URI fileURI = fileURI(modelFile);
		Resource emfResource = resourceSet.createResource(fileURI);
		return emfResource;
	}

	/**
	 * @param modelFile
	 * @return
	 */
	protected URI fileURI(IFile modelFile) {
		URI fileURI = URI.createPlatformResourceURI(modelFile.getFullPath().toString());
		return fileURI;
	}

	/**
	 * creates an array of uris from a list of models
	 * 
	 * @param modelList
	 * @return
	 */
	protected String[] convertModelListToUriArray(List<Model> modelList) {
		String[] strArray = new String[modelList.size()];
		int i = 0;
		for (Model model : modelList) {
			strArray[i] = model.getUri();
			i++;
		}
		return strArray;
	}

	/**
	 * returns all model conforming to the metamodel uris
	 * 
	 * @param artifacts
	 * @param mmUri
	 * @return
	 */
	protected List<Model> findAllModels(EList<Artifact> artifacts, String... mmUri) {
		List<Model> modelList = new ArrayList<Model>();
		for (String string : mmUri) {
			List<Model> models = findAllModels(artifacts, string);
			modelList.addAll(models);
		}
		return modelList;
	}

	/**
	 * returns all model conforming to the metamodel uri
	 * 
	 * @param artifacts
	 * @param mmUri
	 * @return
	 */
	private List<Model> findAllModels(EList<Artifact> artifacts, String mmUri) {
		List<Model> modelList = new ArrayList<Model>();
		if (artifacts != null && mmUri != null)
			for (Artifact artifact : artifacts) {
				if (artifact instanceof Model) {
					Model model = (Model) artifact;
					if (model.getMm() != null)
						if ((mmUri.equals(model.getMm().getUri()))
								|| ((URI_CONSTANT + mmUri).equals(model.getMm().getUri())))
							modelList.add(model);
				}
			}
		return modelList;
	}

	/**
	 * Returns the first real artifact in the list.
	 * 
	 * @param artifacts
	 * @return
	 */
	protected Artifact getFirstArtifact(EList<Artifact> artifacts) {
		for (Artifact artifact : artifacts) {
			if (isThisClazz(artifact.getClass(), Artifact.class))
				return artifact;
		}
		return null;
	}

	protected List<Artifact> getAllArtifacts(EList<Artifact> artifacts) {
		List<Artifact> list = new ArrayList<Artifact>();
		for (Artifact artifact : artifacts) {
			if (isThisClazz(artifact.getClass(), Artifact.class)) {
				list.add(artifact);
			}
		}
		return list;
	}

	protected List<Model> getAllModels(EList<Artifact> artifacts) {
		List<Model> list = new ArrayList<Model>();
		for (Artifact artifact : artifacts) {
			if (isThisClazz(artifact.getClass(), Model.class)) {
				list.add((Model) artifact);
			}
		}
		return list;
	}

	/**
	 * checks if the class class implements the interface or a sub interface
	 * @param theClass
	 * @return
	 */
	@SuppressWarnings("unchecked")
	protected boolean isThisClazz(Class theClass, Class theInterface) {
		Class<?>[] interfaces = theClass.getInterfaces();
		for (Class<?> clazz : interfaces) {
			if (clazz == theInterface)
				return true;
			else if (clazz.getSuperclass() != null)
				return isThisClazz(clazz.getSuperclass(), theInterface);
		}
		return false;
	}
}