package de.urszeidler.eclipse.callchain.tabbedpropery.basicsections;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecoretools.tabbedproperties.sections.AbstractTextPropertySection;
import org.eclipse.emf.ecoretools.tabbedproperties.utils.TextChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

public abstract class AbstractTextButtonSection extends AbstractTextPropertySection {

    //private CLabel fontLabel;
    private Button button;
    private Text text;
    private CLabel label;
    private TextChangeListener listener;

    public void createControls(Composite parent, TabbedPropertySheetPage tabbedPropertySheetPage)
    {
       super.createControls(parent, tabbedPropertySheetPage);
//        Composite composite = getWidgetFactory().createFlatFormComposite(parent);
//        
//        createWidgets(composite);
//        
//         setFormData(composite);
    }
    
    /**
     * handle the button pressed
     *
     */
    protected abstract void handleButtonPressed();
    /**
     * Creates the label widget and the text field control
     * 
     * @param composite The composite holding both widgets
     */
    protected void createWidgets(Composite composite)
    {
    	label = getWidgetFactory().createCLabel(composite, getLabelText());
        text = getWidgetFactory().createText(composite, "", getStyle());

        if (getFeature() != null)
        {
            boolean isChangeable = getFeature().isChangeable();

            text.setEditable(isChangeable);
            text.setEnabled(isChangeable);
        }

         
        //fontLabel = getWidgetFactory().createCLabel(composite, getLabelText());

        //fontText = getWidgetFactory().createText(composite, "");
        //getText().setEditable(false);

        button = getWidgetFactory().createButton(composite, "Change...", SWT.PUSH);

 

    }
    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#hookListeners()
     */
    protected void hookListeners()
    {
        button.addSelectionListener(new SelectionListener()
        {
            public void widgetSelected(SelectionEvent event)
            {
                handleButtonPressed();
                handleTextModified();
            }

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				handleButtonPressed();
				
			}
        });
        
        text.addListener(SWT.Modify, new Listener()
        {
            public void handleEvent(Event e)
            {
                verifyField(e);
            }
        });

        listener = new TextChangeListener()
        {
            public void textChanged(Control control)
            {
                handleTextModified();
            }

            public void focusIn(Control control)
            {
                AbstractTextButtonSection.this.focusIn();
            }

            public void focusOut(Control control)
            {
                AbstractTextButtonSection.this.focusOut();
            }
        };
        listener.startListeningTo(text);

        // Do not listen to the "Enter" key pressed when the widget is a
        // MultiLine Text
        if ((text.getStyle() & SWT.MULTI) == 0)
        {
            listener.startListeningForEnter(text);
        }

    }

    protected void setSectionData(Composite composite)
    {
        FormData data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.right = new FormAttachment(text, -ITabbedPropertyConstants.HSPACE);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        label.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0, getStandardLabelWidth(composite, new String[] {getLabelText()}));
        data.right = new FormAttachment(button, -ITabbedPropertyConstants.HSPACE);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        text.setLayoutData(data);

        data = new FormData();
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(label, ITabbedPropertyConstants.VSPACE, SWT.CENTER);
        button.setLayoutData(data);

    }


    protected void setFormData(Composite composite) {
        FormData data;
        
        data = new FormData();
        data.left = new FormAttachment(0, getStandardLabelWidth(composite, new String[] {getLabelText()}));
        data.right = new FormAttachment(button, -ITabbedPropertyConstants.HSPACE);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        text.setLayoutData(data);

        
        data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.right = new FormAttachment(text, -ITabbedPropertyConstants.HSPACE);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        label.setLayoutData(data);


 
        data = new FormData();
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(label, ITabbedPropertyConstants.VSPACE, SWT.CENTER);
        button.setLayoutData(data);
    }
    
    /**
     * Handle the text modified event. When there is any problem while creating and executing the command, the view will
     * be only refreshed
     */
    protected void handleTextModified()
    {
        if (isTextValid())
        {
            createCommand(getOldFeatureValue(), getNewFeatureValue(getText().getText()));
        }
        else
        {
            refresh();
        }
    }

    /**
     * Get the style of the text widget. By default, this is a single line text
     * 
     * @return the style
     */
    protected int getStyle()
    {
        return SWT.SINGLE;
    }
//    /**
//     * @return the listener
//     */
//    protected TextChangeListener getListener()
//    {
//        return listener;
//    }

    /**
     * @return the text
     */
    protected Text getText()
    {
        return text;
    }

    /**
     * @return the nameLabel
     */
    public CLabel getNameLabel()
    {
        return label;
    }

    /**
     * Check whether the text entered is valid or not. Subclasses should override this method to provide their own
     * check. By default, return true.
     * 
     * @return true if the text entered is valid
     */
    protected boolean isTextValid()
    {
        return true;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTextPropertySection#verifyField(Event)
     */
    protected void verifyField(Event e)
    {
        // do nothing
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTextPropertySection#getFeatureAsString()
     */
    protected String getFeatureAsString()
    {
        String string = getEObject() == null ? null : (String) getEObject().eGet(getFeature());
        if (string == null)
        {
            return "";
        }
        return string;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTextPropertySection#getNewFeatureValue(java.lang.String)
     */
    protected Object getNewFeatureValue(String newText)
    {
        return newText;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTextPropertySection#getOldFeatureValue()
     */
    protected Object getOldFeatureValue()
    {
        return getFeatureAsString();
    }
    /**
     * Handle action when the focus is gained Default action is to do nothing. Clients may override this method if they
     * desire a particular action to be executed when the text control gain the focus.
     */
    protected void focusIn()
    {
        // Do nothing
    }
    public void refresh()
    {
        getText().setText(getFeatureAsString());
    }
    /**
     * Handle action when the focus is lost. Default action is to do nothing. Clients may override this method if they
     * desire a particular action to be executed when the text control lost the focus.
     */
    protected void focusOut()
    {
        // Do nothing
    }

     protected abstract String getLabelText() ;

     protected  abstract EStructuralFeature getFeature() ;
}
