/**
 * 
 */
package de.urszeidler.eclipse.callchain.topcasedgenerators;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.topcased.generator.AbstractGenerator;
import org.topcased.generator.util.ConfiguratorObjectManager;
import org.topcased.modeler.editorconfigurator.EditorConfiguration;
import org.topcased.modeler.generator.internal.generators.EditorConfiguratorGenerator;

import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.executables.AbstractExecuteable;

/**
 * @author urs
 * 
 */
public class EditorGenerator extends AbstractExecuteable implements GeneratorExecutable {

	private static final String TC_EDITOR_CONFIGURATOR_0_10 = "http://www.topcased.org/editorconfigurator/0.10";
	private static final String TC_EDITOR_CONFIGURATOR_1_0 = "http://www.topcased.org/editorconfigurator/1.0";

	/**
	 * 
	 */
	public EditorGenerator() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	@Override
	public void configure(Generic_Generator generator) {
		super.configure(generator);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void execute(IProgressMonitor monitor) throws CoreException {
		Model model = findModelByMetaModel(generator.getUses(), TC_EDITOR_CONFIGURATOR_0_10, TC_EDITOR_CONFIGURATOR_1_0);
		if (model == null)
			return;

		String uri = model.getUri();
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		IFile currentFile = wksroot.getFile(new Path(uri));
		EditorConfiguration configuration = null;

		try {

			ConfiguratorObjectManager manager = new ConfiguratorObjectManager();
			manager.load(currentFile.getFullPath());
			// retrieve the root model object and check if it has the
			// expected type
			if (manager.getRootModelObject() instanceof EditorConfiguration) {
				configuration = (EditorConfiguration) manager.getRootModelObject();
				EditorConfiguratorGenerator generator = new EditorConfiguratorGenerator(configuration);
				IProject generatedProject = generator.generate(monitor);

				if (configuration != null && generatedProject != null)
					if (configuration.isOrganizeImports())
						AbstractGenerator.organizeImports(generatedProject);

			}
		} catch (Exception e) {
			throw new CoreException(new Status(IStatus.ERROR, "de.urszeidler.eclipse.callchain.topcasedgenerators",
					"error while generating editor pluging", e));
		}

	}

}
