/**
 * 
 */
package de.urszeidler.eclipse.callchain.topcasedgenerators;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.DiagramLink;
import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.Reference;
import org.topcased.modeler.di.model.SemanticModelBridge;
import org.topcased.modeler.diagrams.model.Diagrams;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.executables.AbstractExecuteable;

/**
 * @author urs
 * 
 */
public class TopcasedDiagramExchanger extends AbstractExecuteable implements GeneratorExecutable {
	private static final String DOT = "::";
	private final String TOPCASED_DIA_1_0 = "http://www.topcased.org/Diagrams/1.0";
	private List<EObject> containers = new ArrayList<EObject>();
	private ResourceSet resourceSet;
	private List<EObject> listToRemove = new ArrayList<EObject>();
	private Map<EMFSemanticModelBridge, EObject> semanticModelMap = new HashMap<EMFSemanticModelBridge, EObject>();
	private Map<Diagrams, EObject> diagramsMap = new HashMap<Diagrams, EObject>();
	private URI targetUri;
	private Set<String> typesToCopy= new HashSet<String>(); 
	/**
	 * 
	 */
	public TopcasedDiagramExchanger() {
		typesToCopy.add("Usage");
		typesToCopy.add("Comment");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	@Override
	public void configure(Generic_Generator generator) {
		super.configure(generator);
		resourceSet = createResourceSet();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void execute(IProgressMonitor monitor) {
		Model diamodel = findModelByMetaModel(generator.getUses(), TOPCASED_DIA_1_0);
		List<Model> models = getAllModels(generator.getUses());
		models.remove(diamodel);
		monitor.beginTask("loadin models", models.size() + 10);
		if(generator.getProduce().size()>0) {
			Artifact artifact = generator.getProduce().get(0);
			targetUri = artifactToUri(artifact);
		}
		
		Object eObject = loadEObject(diamodel, resourceSet);
		for (Model model2 : models) {
			monitor.subTask("loading :" + model2.getName());
			Object object = loadEObject(model2, resourceSet);
			if (object instanceof List) {
				List l = (List) object;
				containers.addAll(l);

			} else
				containers.add((EObject) object);

			monitor.worked(1);
		}

		if (eObject instanceof Diagrams) {
			Diagrams diagrams = (Diagrams) eObject;
			monitor.subTask("processing diagram, collecting the elements");
			processDiagrams(diagrams);
			monitor.worked(8);
			monitor.subTask("removeing elements");
			processElements(monitor);
			removeElements();
			monitor.worked(1);

			try {

				Resource targetResource = resourceSet.createResource(targetUri);

				monitor.subTask("saving diagram");
				Map options = new HashMap();
				targetResource.getContents().addAll(containers);
				targetResource.save(options);
				diagrams.eResource().save(options);
				monitor.worked(1);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

	private void processElements(IProgressMonitor monitor) {
		monitor.subTask("processing diagrams");
		Set<Diagrams> diagrams = diagramsMap.keySet();
		for (Diagrams diagramsi : diagrams) {
			EObject eobject = diagramsMap.get(diagramsi);
			Resource resource = diagramsi.getModel().eResource();
//			targetUri = resource.getURI();
//			System.out.println(targetUri);
			diagramsi.setModel(eobject);
//			System.out.println("setting :"+diagramsi+"  to model "+eobject );
		}

		monitor.subTask("processing elemnts");
		Set<EMFSemanticModelBridge> smodels = semanticModelMap.keySet();
		for (EMFSemanticModelBridge semanticModelBridge : smodels) {
			EObject object = semanticModelMap.get(semanticModelBridge);
			if (object != null) {
				semanticModelBridge.setElement(object);
			} else {
				// if(inNewModel.eIsProxy())
				// EcoreUtil.resolveAll(inNewModel);
				String eclassname = semanticModelBridge.getElement().eClass().getName();
				System.out.println("checking :"+buildQualifiedName(semanticModelBridge.getElement()));
				
				if (typesToCopy.contains(eclassname))
					copyElement(semanticModelBridge);
				else
					listToRemove.add(semanticModelBridge.getGraphElement());
			}

		}

	}

	/**
	 * 
	 */
	private void removeElements() {
		System.out.println("removing :" + listToRemove);
		ArrayList<EObject> list = new ArrayList<EObject>();
		for (EObject object : listToRemove) {
			// EObject container = object.eContainer();
			if (object != null) {
				if (object instanceof GraphElement) {
					GraphElement ge = (GraphElement) object;
					list.add(ge);
					EList<GraphConnector> anchorage = ge.getAnchorage();

					list.addAll(anchorage);
					for (GraphConnector graphConnector : anchorage) {
						EList<GraphEdge> graphEdge = graphConnector.getGraphEdge();
						list.addAll(graphEdge);

					}
					// ge.getContained().remove(object);
				}
				// EcoreUtil.delete(graphConnector);

			}
		}
//		System.out.println("removing :" + list);
		for (EObject object : list) {
			//System.out.println("-----> " + object);
			EcoreUtil.delete(object);
		}
	}

	/**
	 * @param eObject
	 */
	private void processDiagrams(Diagrams diagrams) {
		exchangeDiagramsModel(diagrams);
		
		
		EList<Diagram> diagrams2 = diagrams.getDiagrams();
		for (Diagram diagram : diagrams2) {
			processDiagram(diagram);
			
		}
		EList<Diagrams> subdiagrams = diagrams.getSubdiagrams();
		for (Diagrams subdiagram : subdiagrams) {
			processDiagrams(subdiagram);
			
		}

	}

	private void processDiagram(Diagram diagram) {
//		System.out.println("----> process diagram :" + diagram.getName());
		exchangeSemanticModel(diagram.getSemanticModel());
		
		
		EList<DiagramLink> diagramLinks = diagram.getDiagramLink();
		for (DiagramLink diagramLink : diagramLinks) {
			GraphElement graphElement = diagramLink.getGraphElement();

			processGraphElements(graphElement);

		}

		EList<DiagramElement> list = diagram.getContained();
		for (DiagramElement diagramElement : list) {
			processDiagramElement(diagramElement);
		}
	}

	private void processDiagramElement(DiagramElement diagramElement) {
		//System.out.println("process diagramelement :" + diagramElement);
		EList<Reference> reference = diagramElement.getReference();
		for (Reference reference2 : reference) {
			DiagramElement referenced = reference2.getReferenced();
			processDiagramElement(referenced);
		}

		if (diagramElement instanceof Diagram) {
			Diagram dia = (Diagram) diagramElement;
			processDiagram(dia);
		} else if (diagramElement instanceof GraphElement) {
			GraphElement ge = (GraphElement) diagramElement;
			processGraphElements(ge);
		}

	}

	private void processGraphElements(GraphElement graphElement) {
		//System.out.println("process grapelement :" + graphElement);
		SemanticModelBridge semanticModel = graphElement.getSemanticModel();
		exchangeSemanticModel(semanticModel);
		EList<DiagramElement> list = graphElement.getContained();
		for (DiagramElement diagramElement : list) {
			processDiagramElement(diagramElement);
		}

	}

	private void exchangeDiagramsModel(Diagrams diagrams) {
		EObject model = diagrams.getModel();
		EObject newEObject = findEObjectInNewModel(model);
		//System.out.println("diagrams : --------> "+model + "<----->" + newEObject);
		diagramsMap.put(diagrams, newEObject);
		// diagrams.setModel(newEObject);
	}

	private void exchangeSemanticModel(SemanticModelBridge semanticModel) {
		// System.out.println(semanticModel);
		if (semanticModel instanceof EMFSemanticModelBridge) {
			EMFSemanticModelBridge emfsmb = (EMFSemanticModelBridge) semanticModel;
			EObject element = emfsmb.getElement();
			if (element.eIsProxy())
				EcoreUtil.resolveAll(element);

			EObject inNewModel = findEObjectInNewModel(element);
			semanticModelMap.put(emfsmb, inNewModel);
			//			
			// System.out.println(element + "<----->" + inNewModel);
			// if (inNewModel != null) {
			// emfsmb.setElement(inNewModel);
			// } else {
			// // if(inNewModel.eIsProxy())
			// // EcoreUtil.resolveAll(inNewModel);
			// copyElement(emfsmb, element);
			// }
		}

	}

	/**
	 * @param emfsmb
	 * @param element
	 */
	private void copyElement(EMFSemanticModelBridge emfsmb) {
		GraphElement graphElement = emfsmb.getGraphElement();
		EList<GraphConnector> anchorage = graphElement.getAnchorage();

		EObject element = emfsmb.getElement();
		EObject object = EcoreUtil.copy(element);
		insertObjectInContainer(element, object);
		emfsmb.setElement(object);
	}

	@SuppressWarnings("unchecked")
	private void insertObjectInContainer(EObject element, EObject object) {
		String qualifiedName = buildQualifiedName(element);
		String[] path = qualifiedName.split(DOT);
		EObject nextcontainer = null;
		;

		for (EObject container : containers) {
			nextcontainer = container;
			for (int i = 0; i < path.length - 1; i++) {
				String pathname = path[i];
				EObject lastcontainer = nextcontainer;
				nextcontainer = findByName(lastcontainer, pathname);
				if (nextcontainer == null)
					break;
			}

			if (nextcontainer != null) {
				String p = path[path.length - 1];
				String[] split = p.split("[.]");
				int pos=0;
				if(split.length==2)
					pos = Integer.parseInt(split[1]);
				// EStructuralFeature containingFeature =
				// nextcontainer.eContainingFeature();
				Object l = nextcontainer.eGet(element.eContainingFeature());

				if (l instanceof List) {
					try {
						List list = (List) l;
						if (list.size()-1 > pos)
							list.add(pos, object);
						else
							list.add(object);

					} catch (Exception e) {
						System.out.println("error while addin to :" + nextcontainer + "  " + l);
					}
				}
				break;
			}
		}
	}

	private EObject findEObjectInNewModel(EObject element) {
		String qualifiedName = buildQualifiedName(element);
		return lookupEObjectByQuallifiedName(qualifiedName);

	}

	private EObject lookupEObjectByQuallifiedName(String qualifiedName) {
		// System.out.println("lookin up --------> " + qualifiedName);
		if (qualifiedName == null)
			return null;

		String[] path = qualifiedName.split(DOT);
		qualifiedName.split(DOT);
		if (path.length == 0)
			path = new String[] { qualifiedName };

		EObject nextcontainer = null;
		for (EObject container : containers) {

			nextcontainer = findByQuallifiedName(path, container);
			if (nextcontainer != null)
				return nextcontainer;
		}

		return nextcontainer;
	}

	/**
	 * @param path
	 * @param nextcontainer
	 * @param container
	 * @return
	 */
	private EObject findByQuallifiedName(String[] path, EObject container) {
		EObject nextcontainer = container;
		for (String pathname : path) {
			EObject lastcontainer = nextcontainer;
			nextcontainer = findByName(lastcontainer, pathname);
			// System.out.println("--> " + nextcontainer+"  -- "+pathname);
			if (nextcontainer == null) {
				// System.out.println( path[path.length-1]+"  "+" not found");
				// return null;
				return null;
			}
		}
		// if (nextcontainer != null)
		// return nextcontainer;
		// System.out.println("findByQuallifiedName in container :" +
		// nextcontainer);
		return nextcontainer;
	}

	/**
	 * @param container
	 * @param pathname
	 * @return
	 */
	private EObject findByName(EObject container, String pathname) {
		EObject nextcontainer;
		nextcontainer = findInContainer(container, pathname);
		// System.out.println("-------------> :" + pathname + " ---> " +
		// nextcontainer);
		return nextcontainer;
	}

	private EObject findInContainer(EObject container, String pathname) {
		// System.out.println("looking in container :" + pathname);
		if (hasThisName(pathname, container))
			return container;
		EList<EObject> contents = container.eContents();
		for (EObject object : contents) {
			if (hasThisName(pathname, object)) {
				// System.out.println("object found ---------------> " +
				// object);
				return object;
			}
		}
		return null;
	}

	/**
	 * @param pathname
	 * @param object
	 */
	private boolean hasThisName(String pathname, EObject object) {
		String name = findNameForEObject(object);
		if (name != null)
			return (name.equals(pathname));
		// return object;
		return false;
	}

	private String buildQualifiedName(EObject element) {
		String nameForEObject = findNameForEObject(element);
		if (nameForEObject != null) {
			StringBuffer name = new StringBuffer(nameForEObject);

			EObject container = element.eContainer();
			if (container != null)
				name.insert(0, DOT);
			while (container != null) {
				String containerName = findNameForEObject(container);
				container = container.eContainer();
				if (containerName != null) {
					name.insert(0, containerName);
					if (container != null)
						name.insert(0, DOT);
				}
			}
			//System.out.println("--->" + name.toString());
			return name.toString();
		}
		//System.out.println("--->" + nameForEObject);
		return nameForEObject;
	}

	private String findNameForEObject(EObject element) {
		EClass eclass = element.eClass();
		EList<EAttribute> attributes = eclass.getEAllAttributes();
		// checking for a name property
		for (EAttribute attribute : attributes) {
			if ("name".equals(attribute.getName())) {
				if (element.eGet(attribute) == null)
					return "<unset>"+buildContainerIndexName(element);

				return element.eGet(attribute).toString();
			}
		}
		// take the index of the container
		if (element.eContainer() == null)
			return null;

		return buildContainerIndexName(element);
	}

	/**
	 * @param element
	 * @return
	 */
	private String buildContainerIndexName(EObject element) {
		if(element.eContainer() ==null)
			return "";
			
		Integer index = element.eContainer().eContents().indexOf(element);
		return element.eClass().getName() + "." + index.toString();
	}

}
